#define _GNU_SOURCE

#include <browser.h>
#include <Ecore_Evas.h>
#include <etk/Etk.h>
#include <debug.h>
#include <packagekit/pk-package-id.h>
#include <string.h>
#include <assassin.h>
#include <gears.h>
#include <confirm_command.h>

extern  Ecore_Evas  *   ee;
Evas_List *category_list=NULL;
enum ActionType get_actiontype();
static Etk_Bool categories_selected(Etk_Object *object, Etk_Tree_Row *row, Etk_Event_Mouse_Up *event, void *data);
static Etk_Bool packages_selected(Etk_Object *object, Etk_Tree_Row *row, Etk_Event_Mouse_Up *event, void *data) ;

struct _categories {
	const char * title;
	PkGroupEnum group;
};

struct _categories CATEGORIES[] = {
	{"Communication", 	PK_GROUP_ENUM_COMMUNICATION}, 
	{"Development",		PK_GROUP_ENUM_PROGRAMMING}, 
	{"Games", 		PK_GROUP_ENUM_GAMES}, 
	{"Maps", 		PK_GROUP_ENUM_MAPS}, 
	{"Miscellaneous", 	PK_GROUP_ENUM_UNKNOWN}, 
	{"Network", 		PK_GROUP_ENUM_NETWORK}, 
	{"Repos", 		PK_GROUP_ENUM_REPOS}, 
	{NULL, 			0}  // The last one is full of NULL pointers
};

static void get_categories_lists (struct Scroll_List *SL) {
	int i=0;
	struct Group_List * GL;
	//ETK_TREE_ROW_CLICKED_SIGNAL
	if (category_list) {
		error("This function shall only run once!\n");
		return;
	}
	for (i=0;CATEGORIES[i].title;i++) {
		GL = get_packages_list(CATEGORIES[i].title,CATEGORIES[i].group,NULL);
		category_list = evas_list_append(category_list,GL);
		GL->row = scroll_list_append(SL,CATEGORIES[i].title);
	}
	etk_signal_connect_by_code(ETK_TREE_ROW_CLICKED_SIGNAL,ETK_OBJECT(SL->tree),&categories_selected,category_list);
}


struct Scroll_Page * get_categories_page () {
	static struct Scroll_Page * SP=NULL;
	struct Scroll_List * SL;
	if (SP) {
		return SP;
	}
	SP = scroll_page_new();
	SL = scroll_list_new(SP);

	scroll_page_set_title(SP,"Categories");
	get_categories_lists(SL);

	etk_widget_show_all(SL->embed);
	current_scroll_page_set(SP);
	return SP;
}


inline int get_GL_index(char *info) {
	const char * AVAILIABLE="available";
	const char * INSTALLED ="installed";
	const int INDEX_INSTALL=0;
	const int INDEX_UPDATE=1;
	const int INDEX_UNINSTALL=2;

	if (!info) return -1;
	if (!strncmp(info,AVAILIABLE,strlen(AVAILIABLE))) {
		return INDEX_INSTALL;
	}else if (!strncmp(info,INSTALLED,strlen(INSTALLED))) {
		return INDEX_UNINSTALL;
	}else if (!strncmp(info,"low",strlen("low")) ||
			!strncmp(info,"enhancement",strlen("enhancement"))) {
		return INDEX_UPDATE;
	}
	return -1;
}

static void package_detail_delete (void *data) {
	struct Package_Detail *package_detail = (struct Package_Detail *) data;
	if (!data) {
		error("data shall not be null!!\n");
		return;
	}
	debug("Try to free package detail.\n");
	etk_widget_hide(package_detail->text_view);
	etk_widget_hide(package_detail->embed);
	evas_object_hide(package_detail->package_view);
	etk_object_destroy(ETK_OBJECT(package_detail->text_view));
	etk_object_destroy(ETK_OBJECT(package_detail->embed));
	evas_object_del(package_detail->package_view);
	free(package_detail);
}

inline struct Target * packageid_to_target (char *packageid) {
	struct Target * T =  (struct Target*) malloc(sizeof(struct Target));
	memset(T, 0, sizeof(*T));
	T->type =get_buttom_bar_selected_type(); 
	T->status = INITIAL;
	T->package = packageid;
	return T;
}

static void package_action (void *data,  Evas_Object *obj, const char *emission, const char *source) {
	char * packageid = (char *)data;
	struct Target * T = packageid_to_target(packageid);
	debug("Try to confirm %d %s\n",T->type,T->package);
	do_confirm(ecore_evas_get(ee),T);
}
static void pk_get_description_cb (void *data, DBusMessage *message) {
	Pk_Operation * pko = (Pk_Operation *)data;
	struct Scroll_Page *SP;
	DBusError _err;
	struct Package_Detail *package_detail;
	char buf[MAX_PACKAGE_NAME_SIZE];
	char name[MAX_PACKAGE_NAME_SIZE];
	int ret;
	char *tid;
	char *packageid;
	char *license;
	char *group_text;
	char *description;
	char *url;
	unsigned int size;
	if (!data) {
		error("data shall not be null!!\n");
		return;
	}
	dbus_error_init(&_err);	
	ret=dbus_message_get_args(message,&_err,
			DBUS_TYPE_STRING, &tid,
			DBUS_TYPE_STRING, &packageid, 
			DBUS_TYPE_STRING, &license, 
			DBUS_TYPE_STRING, &group_text, 
			DBUS_TYPE_STRING, &description, 
			DBUS_TYPE_STRING, &url, 
			DBUS_TYPE_UINT64, &size, 
			DBUS_TYPE_INVALID);
	if (ret==0) {
		error("Error message: %s\n",_err.message);
		dbus_error_free(&_err);
		return;
	}
	dbus_error_free(&_err);
	if (!pk_should_proxy(pko,tid)) {
		return;
	}
	package_detail = (struct Package_Detail *) malloc(sizeof(struct Package_Detail));
	SP = scroll_page_new();
	scroll_page_set_volatilize(SP, 1);
	package_detail->package_view = new_package_view(ecore_evas_get(ee),packageid);
	edje_object_part_swallow(SP->frame, "scroll.frame", package_detail->package_view);
	get_pk_name_from_id(packageid,name,sizeof(name));
	sprintf(buf,"Categories / %s / %s",group_text,name);
	debug("get %s\n",buf);
	scroll_page_set_title(SP,buf);
	package_detail->embed = etk_embed_new (ecore_evas_get(ee));
	package_detail->text_view = etk_text_view_new();

	etk_container_add(ETK_CONTAINER(package_detail->embed), package_detail->text_view);
	edje_object_part_swallow(package_detail->package_view,"package.description.text",etk_embed_object_get(ETK_EMBED(package_detail->embed)));
	etk_textblock_text_set(ETK_TEXT_VIEW(package_detail->text_view)->textblock,description,ETK_FALSE);

	if (size > 1024) {
		sprintf(buf,"Download size: %d KB",(int)size/1024);
	}else {
		sprintf(buf,"Download size: %d Bytes",(int)size);
	}
	edje_object_part_text_set(package_detail->package_view,"package.size.text",buf);
	if (get_buttom_bar_selected_type()==INSTALL) {
		sprintf(buf,"(Touch me to install)");
	}else if (get_buttom_bar_selected_type()==UPDATE) {
		sprintf(buf,"(Touch me to update)");
	}else {
		sprintf(buf,"(Touch me to uninstall)");
	}
	edje_object_part_text_set(package_detail->package_view,"touchme.label.text",buf);

	SP->embed = package_detail;
	SP->free_embed = & package_detail_delete;
	edje_object_signal_callback_add(package_detail->package_view, "mouse,clicked,1","thumbnail", package_action, pko->payload );
	evas_object_move(SP->frame,WIDTH(),0);
	etk_widget_show_all(ETK_WIDGET(package_detail->embed));
	scroll_page_shows(SP);
}


#define LENGTH_NAME_SHOWED 64
static void pk_default_search_group_cb (void *data, DBusMessage *message) {
	DBusError _err;
	char * tid;
	char * info;
	char * packageid;
	char * summary;
	int ret;
	Pk_Operation * pko = (Pk_Operation *)data;
	struct Group_List * GL = (struct Group_List *)pko-> payload;
	struct PKG_Item *pkg_item=NULL;
	char name[LENGTH_NAME_SHOWED];


	dbus_error_init(&_err);
	ret=dbus_message_get_args(message,&_err,
			DBUS_TYPE_STRING, &tid,
			DBUS_TYPE_STRING, &info,
			DBUS_TYPE_STRING, &packageid,
			DBUS_TYPE_STRING, &summary,
			DBUS_TYPE_INVALID);
	if (!ret) {
		error("error message: '%s'\n",_err.message);
		dbus_error_free(&_err);
		return;
	}
	dbus_error_free(&_err);
	if (!pk_should_proxy(pko,tid)) {
		return;
	}
	if (GL==NULL) {
		error("GL shall not be Null now!!\n");
		return;
	}
	ret = get_GL_index(info);
	if (ret < 0) {
		debug("info of package: '%s' is '%s' and assassin do not know that!\n",packageid,info);
		return;
	}
	get_pk_name_from_id(packageid,name,sizeof(name));
	pkg_item = (struct PKG_Item *)malloc(sizeof(struct PKG_Item));

	pkg_item->packageid = strdup(packageid);
	pkg_item->row = scroll_list_append(GL->list[ret],name);
	GL->sublist[ret] = evas_list_append(GL->sublist[ret],pkg_item);
	if (evas_list_count(GL->sublist[ret])==1) {
		etk_signal_connect_by_code(ETK_TREE_ROW_CLICKED_SIGNAL,ETK_OBJECT(GL->list[ret]->tree),&packages_selected,GL->sublist[ret]);
	}
	//debug("Putting %s into %d, list size %d\n",name, ret,evas_list_count(GL->sublist[ret]));
}

static void search_group(PkGroupEnum group, struct Group_List *GL) {
	Pk_Operation *  pko = (Pk_Operation *) malloc(sizeof(Pk_Operation));
	memset(pko,0,sizeof(Pk_Operation));
	debug("Search group %d %s\n",group, pk_group_enum_to_text(group));
	pko->Operations[Package_sig]=&pk_default_search_group_cb;
	pko->payload = GL;
	pk_search_group(pko,pk_group_enum_to_text(group));
}


struct Group_List * get_packages_list(const char *header, PkGroupEnum group, struct Group_List *GL) {
	char buf[MAX_PACKAGE_NAME_SIZE];
	int i;
	if (GL==NULL) {
		GL= (struct Group_List * )malloc(sizeof(struct Group_List));
		memset(GL,0,sizeof(struct Group_List));
		sprintf(buf,"Categories / %s",header);
		for (i=0; i<NUMBER_BUTTOM_LIST; i++) {
			GL->page[i] = scroll_page_new();
			scroll_page_set_title(GL->page[i], buf);
			GL->list[i] =  scroll_list_new(GL->page[i]);
		}
		search_group(group,GL);
	}
	return GL;
}


inline int get_index_in_categories_tree(Evas_List *list, Etk_Tree_Row *row) {
	int i=0;
	for (i=0;i<evas_list_count(list);i++) {
		if (((struct Group_List *)evas_list_nth(list,i))->row==row) {
			return i;
		}
	}
	return -1;
}
static Etk_Bool categories_selected(Etk_Object *object, Etk_Tree_Row *row, Etk_Event_Mouse_Up *event, void *data) {
	struct Group_List *GL;
	int action_type = get_buttom_bar_selected_type();
	Evas_List *gl_list = (Evas_List *) data;
	debug("clicked !! list count=%d\n",evas_list_count(gl_list));
	debug("The %d'th item is selected!!\n",get_index_in_categories_tree(gl_list,row));
	GL = evas_list_nth(gl_list,get_index_in_categories_tree(gl_list,row));

	scroll_page_shows(GL->page[action_type]);	

	return ETK_TRUE;
}

inline int get_index_in_packages_tree(Evas_List *list, Etk_Tree_Row *row) {
	int i=0;
	for (i=0;i<evas_list_count(list);i++) {
		if (((struct PKG_Item *)evas_list_nth(list,i))->row==row) {
			return i;
		}
	}
	return -1;
}

static Etk_Bool packages_selected(Etk_Object *object, Etk_Tree_Row *row, Etk_Event_Mouse_Up *event, void *data) {
	Evas_List *list = (Evas_List *)data;
	struct PKG_Item * pkg_item=NULL;
	Pk_Operation * pko;
	pko = (Pk_Operation *) malloc (sizeof(Pk_Operation));
	memset(pko, 0 , sizeof(Pk_Operation));

	debug("The %d'th item is selected!!\n",get_index_in_packages_tree(list,row));
	pkg_item = (struct PKG_Item * )evas_list_nth(list,get_index_in_packages_tree(list,row));
	debug("packageid = %s\n",pkg_item->packageid);

	pko->payload = pkg_item->packageid;

	pko->Operations[Description_sig] = & pk_get_description_cb;
	pk_get_description (pko,pkg_item->packageid);
	return ETK_TRUE;
}
