#include <stdlib.h>
#include <gears.h>
#include <assassin.h>
#include <debug.h>
#include <browser.h>

#define _GNU_SOURCE
#include <string.h>

extern Ecore_Evas  *   ee;

void set_color(Evas_Object *o, int color, int alpha) {
	int R = (color & 0x00ff0000) >> 16;
	int G = (color & 0x0000ff00) >> 8;
	int B = (color & 0x000000ff) ;
	evas_object_color_set(o, R, G, B, alpha);
}

int WIDTH()  {
	int __width;
	evas_output_viewport_get(ecore_evas_get(ee),NULL,NULL,&__width,NULL);
	return __width;
}
int HEIGHT()  {
	int __height;
	evas_output_viewport_get(ecore_evas_get(ee),NULL,NULL,NULL, &__height);
	return __height;
}


struct Installer_Alert * __new_Alert(Evas *evas, const char *type, const char *message, const char *icon_p, const char *icon_n, CallbackFunc pos, CallbackFunc neg, void *data) {
	struct Installer_Alert * AA = (struct Installer_Alert *)malloc(sizeof(struct Installer_Alert));
	const int alert_height=185;
	const double moving_time=FINISH_TIME_MAGIC;
	int x,y,w,h;
	AA->alert = assassin_edje_object_get("assassin/confirm.alert");
	edje_object_part_text_set(AA->alert,"title",type);
	edje_object_part_text_set(AA->alert,"message",message);
	edje_object_part_text_set(AA->alert,"button.positive",icon_p);
	edje_object_part_text_set(AA->alert,"button.negative",icon_n);
	evas_output_viewport_get(evas,&x,&y,&w,&h);
	evas_object_resize(AA->alert,w,alert_height);	
	animator_move_object (AA->alert,moving_time,x,-alert_height,x,(h-alert_height)/2);	

	// Set Callbacks
	alert_set_callback (AA, pos, neg);

	AA->data = data;
	evas_object_move(AA->alert,-1000,-1000);
	evas_object_show(AA->alert);
	return AA;
}

void dummy_positive_cb (void *data,  Evas_Object *obj, const char *emission, const char *source) {
	debug("Positive!!\n");
}

void dummy_negative_cb (void *data,  Evas_Object *obj, const char *emission, const char *source) {
	debug("Negative!!\n");
}

int alert_set_callback(struct Installer_Alert *AA, CallbackFunc pos, CallbackFunc neg) {
	if (!AA) return 0;
	if (AA->positive && pos != AA->positive) {
		edje_object_signal_callback_del(AA->alert, "mouse,clicked,1","icon.positive", AA->positive);
		debug("remove old callback!!\n");
	}
	edje_object_signal_callback_add(AA->alert, "mouse,clicked,1","icon.positive", pos,AA);
	debug("adding new callback!\n");
	AA->positive = pos;
	if (AA->negative && neg != AA->negative) {
		edje_object_signal_callback_del(AA->alert, "mouse,clicked,1","icon.negative", AA->negative);
	}
	edje_object_signal_callback_add(AA->alert, "mouse,clicked,1","icon.negative", neg,AA);
	AA->negative = neg;
	return 1;
}

struct Installer_Alert * new_Alert (Evas *evas, const char *type, const char *message, const char *icon_p, const char *icon_n, void *data) {
	CallbackFunc  pos = &dummy_positive_cb;
	CallbackFunc  neg = &dummy_negative_cb;
	return __new_Alert(evas, type, message, icon_p, icon_n, pos, neg, data);
}



void free_Alert(struct Installer_Alert *AA) {
	evas_object_del(AA->alert);
	free(AA);
}

struct Installer_Progressbar * new_Progressbar(Evas *evas, const char *text) {
	int w,h;
	struct Installer_Progressbar * IP = (struct Installer_Progressbar *) malloc (sizeof(struct Installer_Progressbar));
	memset(IP,0,sizeof(struct Installer_Progressbar));
	debug("text = %s\n", text);
	IP->bar = assassin_edje_object_get("assassin/action.progressbar");
	progressbar_text_set(IP,text);
	IP->MAX=100;
	IP->WIDTH=WIDTH();
	IP->progress=0;
	progressbar_update(IP, IP->progress);
	edje_object_size_max_get(IP->bar,&w,&h);
	evas_object_resize(IP->bar,w,h);
	evas_object_move(IP->bar,0,(HEIGHT() - h)/2);
	evas_object_show(IP->bar);
	return IP;
}

unsigned int progressbar_increase(struct Installer_Progressbar *AP, int v) {
	AP->progress += v;
	return progressbar_update(AP, AP->progress);
}

static inline void __progressbar_update(struct Installer_Progressbar *AP,unsigned int p) {
	double rate;
	Evas_Object * _bar;
	int progress_height;
	_bar = edje_object_part_object_get(AP->bar,"progressbar");
	if (_bar==NULL) {
		error("Hey progressbar object is gone!!\n");
	}
	evas_object_geometry_get(_bar,NULL,NULL,NULL,&progress_height);
	rate = (double) p / AP->MAX;
	evas_object_resize(_bar, AP->WIDTH*rate, progress_height);
	return;
}

unsigned int progressbar_text_set(struct Installer_Progressbar *AP, const char *text) {
	if (AP->text) {
		free(AP->text);
	}
	if (text==NULL) {
		debug("Text is NULL!!\n");
		return 0;
	}
	AP->text = malloc(32);
	AP->text[0]='\0';
	strncpy(AP->text, text, 32);
	edje_object_part_text_set(AP->bar,"message",AP->text);
	__progressbar_update(AP,AP->progress);
	return 1;
}

unsigned int progressbar_update(struct Installer_Progressbar *AP, unsigned int p) {
	char buf[64];  // Warning there may be buffer overflow issues.
	AP->progress = p;
	AP->progress = AP->progress <= AP->MAX ?  AP->progress : AP->MAX;
	AP->progress = AP->progress >= 0 ?  AP->progress : 0;

	sprintf(buf, "%s: %3d%%", AP->text, AP->progress);
	edje_object_part_text_set(AP->bar,"message",buf);

	__progressbar_update(AP,p);

	return AP->progress;
}



void free_Progressbar(struct Installer_Progressbar *AP) {
	evas_object_del(AP->bar);
	free(AP->text);
	free(AP);
}

struct Buttom_Bottons * BB=NULL;

static void check_BB_status(struct Buttom_Bottons * BB) {
	Evas_Object *obj=NULL;
	obj = edje_object_part_object_get(BB->buttons,"install.text");
	if (BB->selected ==0) {
		evas_object_color_set(obj,0,255,0,255);
	}else {
		evas_object_color_set(obj,255,255,255,255);
	}
	obj = edje_object_part_object_get(BB->buttons,"update.text");
	if (BB->selected ==1) {
		evas_object_color_set(obj,0,255,0,255);
	}else {
		evas_object_color_set(obj,255,255,255,255);
	}
	obj = edje_object_part_object_get(BB->buttons,"uninstall.text");
	if (BB->selected ==2) {
		evas_object_color_set(obj,0,255,0,255);
	}else {
		evas_object_color_set(obj,255,255,255,255);
	}
}

static void BB_install_selected_cb(void *data, Evas_Object *obj, const char *emission, const char *source) {
	struct Buttom_Bottons * BB = (struct Buttom_Bottons *) data;
	BB->selected = 0;
	scroll_page_back_to_root();
	check_BB_status(BB);
}
static void BB_update_selected_cb(void *data, Evas_Object *obj, const char *emission, const char *source) {
	struct Buttom_Bottons * BB = (struct Buttom_Bottons *) data;
	BB->selected = 1;
	scroll_page_back_to_root();
	check_BB_status(BB);
}
static void BB_uninstall_selected_cb(void *data, Evas_Object *obj, const char *emission, const char *source) {
	struct Buttom_Bottons * BB = (struct Buttom_Bottons *) data;
	BB->selected = 2;
	scroll_page_back_to_root();
	check_BB_status(BB);
}




struct Buttom_Bottons *  new_buttom_bar(Evas *evas) {
	const int BAR_HEIGHT=60;

	if (!BB) {
		BB = malloc(sizeof(struct Buttom_Bottons));
	}else {
		return BB;
	}
	BB->buttons =  assassin_edje_object_get("assassin/buttons");	
	evas_object_resize(BB->buttons, WIDTH(), BAR_HEIGHT);
	evas_object_move(BB->buttons, 0, HEIGHT() - BAR_HEIGHT);
	evas_object_show(BB->buttons);
	BB->selected=0;
	edje_object_signal_callback_add(BB->buttons, "mouse,clicked,1","install.buttom", &BB_install_selected_cb,BB);
	edje_object_signal_callback_add(BB->buttons, "mouse,clicked,1","update.buttom", &BB_update_selected_cb,BB);
	edje_object_signal_callback_add(BB->buttons, "mouse,clicked,1","uninstall.buttom", &BB_uninstall_selected_cb,BB);
	check_BB_status(BB);
	return BB;
}

int get_buttom_bar_selected_type() {
	if (!BB) {
		error("BB is null!!\n");
		return 0;
	}
	return BB->selected;
}

#include <packagekit/pk-package-id.h>
Evas_Object * new_package_view (Evas * evas,char *packageid) {
	PkPackageId *pkid;
	Evas_Object *obj=NULL;	

	if (!pk_package_id_check(packageid)) {
		error("Bad Packageid %s!!\n",packageid);
		return NULL;
	}
	pkid     = pk_package_id_new_from_string(packageid);
	obj = assassin_edje_object_get("assassin/package.detail");	
	if (!obj) {
		error("Create obj failed!!\n");
		return NULL;
	}
	edje_object_part_text_set(obj,"package.name.text", (char *)pkid->name);
	edje_object_part_text_set(obj,"package.version.text", (char *)pkid->version);
	edje_object_part_text_set(obj,"package.description.text", (char *)pkid->data);

	pk_package_id_free(pkid);
	return obj;
}



static int __animator_square_move_object(void *object_shift) {
	struct Object_Shift  *OS = (struct Object_Shift  *) object_shift;
	double elapse = ecore_time_get() - OS->start_time;
	double ratio;
	double x,y;
	if (elapse >= OS->finish_time) {
		evas_object_move(OS->obj,OS->e_x,OS->e_y);
		free(OS);
		return 0;
	}
	ratio = elapse / OS->finish_time;
	ratio =  (ratio*ratio);
	x = OS->s_x + ratio * (OS->e_x - OS->s_x);
	y = OS->s_y + ratio * (OS->e_y - OS->s_y);
	evas_object_move(OS->obj,x,y);
	return 1;
}

void animator_move_object(Evas_Object *o,double time,int sx,int sy,int ex,int ey) {
	struct Object_Shift  *OS;
	if (o==NULL) {
		debug("o is NULL!!\n");
		return;
	}
	if (time <= 0) {
		evas_object_move(o,ex,ey);
		return;
	}
       	OS = (struct Object_Shift *) malloc (sizeof(struct Object_Shift));
	OS->obj=o;
	OS->finish_time=time;
	OS->s_x=sx;
	OS->s_y=sy;
	OS->e_x=ex;
	OS->e_y=ey;
	OS->start_time = ecore_time_get();
	ecore_animator_add(__animator_square_move_object,OS);
}


Evas_Object * assassin_edje_object_get(const char *name) {
	char buf[64];
	Evas_Object *obj;
	sprintf(buf,"%s/assassin.edj",THEME_PATH);
	if (!name) {
		error("Name is NULL, what object you want to get??\n");
		return NULL;
	}
	obj = edje_object_add(ecore_evas_get(ee));
	edje_object_file_set(obj, buf, name);
	return obj;
}

struct Scroll_Page * scroll_page_new () {
	struct Scroll_Page * SP = (struct Scroll_Page *) malloc(sizeof(struct Scroll_Page));
	char buf[64];
	memset(SP,0,sizeof(struct Scroll_Page));

	sprintf(buf,"%s/assassin.edj",THEME_PATH);
	
	SP->frame = edje_object_add(ecore_evas_get(ee));
	edje_object_file_set(SP->frame,buf, "assassin/v.scroll");
	SP->next=SP->prev = SP;
	return SP;
}

void scroll_page_set_volatilize(struct Scroll_Page *sp, int v) {
	sp->volatilize=1;
}
void scroll_page_set_title(struct Scroll_Page *sp, char *title) {
	if (!sp) {
		error("SP is NULL!!\n");
		return;
	}
	if (!title) {
		title="";
	}
	edje_object_part_text_set(sp->frame,"title",title);
}


void scroll_page_del (struct Scroll_Page *sp) {
	if (!sp) return;
	if (sp->embed && sp->free_embed)  {
		sp->free_embed(sp->embed);
		sp->embed=NULL;
	}
	evas_object_del(sp->frame);
	sp->frame=NULL;
	free(sp);
}


enum Move_Direction {
	MOVE_TO_LEFT=0,
	MOVE_TO_RIGHT,
	MOVE_TO_UP,
	MOVE_TO_DOWN,
};
struct GroupMove {
	int direction;
	struct Scroll_Page *old_obj;
	struct Scroll_Page *new_obj;
	double finish_time;
	double start_time;
};

void __page_size_adjustion(struct Scroll_Page *SP) ;
static Ecore_Animator *__EA__=NULL; 
static int __scroll_move_animator(void *data) {
	struct GroupMove *gm = (struct GroupMove *)data;
	double elapse = ecore_time_get() - gm->start_time;
	double ratio;
	double offset;
	if (elapse >= gm->finish_time) {
		evas_object_hide(gm->old_obj->frame);
		current_scroll_page_set(gm->new_obj);
		if (gm->old_obj->volatilize) {
			gm->new_obj->next = gm->new_obj->next == gm->old_obj ? NULL : gm->new_obj->next;
			gm->new_obj->prev = gm->new_obj->prev == gm->old_obj ? NULL : gm->new_obj->prev;
			scroll_page_del(gm->old_obj);
		}
		free(gm);
		__EA__=NULL;
		return 0;
	}
	ratio = elapse / gm->finish_time;
	ratio = ratio  * ratio;
	if (gm->direction == MOVE_TO_LEFT) {
		offset = WIDTH() * ratio;
		evas_object_move(gm->old_obj->frame,(int)0-offset,0);
		evas_object_move(gm->new_obj->frame, (int)WIDTH() - offset,0);
	} else if (gm->direction == MOVE_TO_RIGHT) {
		offset =  WIDTH() * ratio;
		evas_object_move(gm->old_obj->frame, offset , 0);
		evas_object_move(gm->new_obj->frame,offset-WIDTH(),0);
	} else if (gm->direction == MOVE_TO_UP) {
		offset = HEIGHT() * ratio;
		evas_object_move(gm->old_obj->frame,0, -offset);
		evas_object_move(gm->new_obj->frame,0, HEIGHT()-offset);
	} else {  // DOWN
		offset = HEIGHT() * ratio;
		evas_object_move(gm->old_obj->frame,0, offset);
		evas_object_move(gm->new_obj->frame,0, offset - HEIGHT());
	}
	return 1;
}
void scroll_page_back_to_root() {
	struct GroupMove *gm;
	struct Scroll_Page *sp = current_scroll_page_get();
	if (__EA__) {
		debug("_EA_ is runinng, skip this request!!\n");
		return;
	}
	if ( sp->prev== NULL || sp->prev == sp) {
		get_categories_page();
		error("Hun!!\n");
		return;
	}
	gm = (struct GroupMove *) malloc(sizeof(struct GroupMove));
	memset(gm,0,sizeof(struct GroupMove));

	gm->direction = MOVE_TO_RIGHT;
	gm->old_obj = sp;
	gm->new_obj = get_categories_page() ;
	__page_size_adjustion(gm->new_obj);
	gm->finish_time = FINISH_TIME_MAGIC;
	gm->start_time =  ecore_time_get();
	__EA__=ecore_animator_add(__scroll_move_animator,gm);
}

void scroll_page_back() {
	struct GroupMove *gm;
	struct Scroll_Page *sp = current_scroll_page_get();
	if (__EA__) {
		debug("_EA_ is runinng, skip this request!!\n");
		return;
	}

	if ( sp->prev== NULL || sp->prev == sp) {
		error("Hun!!\n");
		return;
	}
	gm = (struct GroupMove *) malloc(sizeof(struct GroupMove));
	memset(gm,0,sizeof(struct GroupMove));

	gm->direction = MOVE_TO_RIGHT;
	gm->old_obj = sp;
	gm->new_obj = sp->prev;
	__page_size_adjustion(gm->new_obj);
	gm->finish_time = FINISH_TIME_MAGIC;
	gm->start_time =  ecore_time_get();
	__EA__=ecore_animator_add(__scroll_move_animator,gm);
}

void scroll_page_shows(struct Scroll_Page *sp) {
	struct GroupMove *gm;
	if (__EA__) {
		debug("_EA_ is runinng, skip this request!!\n");
		return;
	}
	if (!sp) {
		error("Hey!! Don't play me!! sp is NULL\n");
		return;
	}
	if (sp == current_scroll_page_get()) {
		current_scroll_page_set(sp); 
		return;
	}
	gm = (struct GroupMove *) malloc(sizeof(struct GroupMove));
	memset(gm,0,sizeof(struct GroupMove));

	gm->direction = MOVE_TO_LEFT;
	gm->old_obj = current_scroll_page_get();
	gm->new_obj = sp;
	
	gm->old_obj->next = gm->new_obj;
	gm->new_obj->prev = gm->old_obj;
	__page_size_adjustion(gm->new_obj);
	
	gm->finish_time = FINISH_TIME_MAGIC;
	gm->start_time =  ecore_time_get();
	__EA__=ecore_animator_add(__scroll_move_animator,gm);
}

struct Scroll_Page * __current_scroll_page__;
struct Scroll_Page *current_scroll_page_get() {
	return __current_scroll_page__;
}
void current_scroll_page_set(struct Scroll_Page *SP) {
	struct Buttom_Bottons * B = new_buttom_bar(ecore_evas_get(ee));
	__current_scroll_page__ = SP;
	evas_object_move(SP->frame,0,0);
	__page_size_adjustion(SP);
	evas_object_raise(B->buttons);  // make the buttons on the top
}
void __page_size_adjustion(struct Scroll_Page *SP) {
	int x,y,w,h;
	evas_output_viewport_get(ecore_evas_get(ee),&x,&y,&w,&h);
	evas_object_resize(SP->frame,w,h);
	evas_object_show(SP->frame);
}

Etk_Tree_Row *  scroll_list_append(struct Scroll_List * SL, const char * title) {
	Etk_Tree_Row *tree_row;
	tree_row=etk_tree_row_append(ETK_TREE(SL->tree),NULL,SL->col, title, NULL );
	etk_tree_thaw(ETK_TREE(SL->tree));
	return tree_row;
}

struct Scroll_List * scroll_list_new(struct Scroll_Page * SP) {
	struct Scroll_List * SL = (struct Scroll_List *)malloc (sizeof(struct Scroll_List));
	memset(SL,0, sizeof(struct Scroll_List));
	int h,w;

	SL->embed  = etk_embed_new(ecore_evas_get(ee));
	SL->scroll = etk_vbox_new(ETK_FALSE, 0);
	SL->tree  = etk_tree_new();
	etk_tree_mode_set(ETK_TREE(SL->tree), ETK_TREE_MODE_LIST);
	etk_tree_multiple_select_set(ETK_TREE(SL->tree), ETK_FALSE);
	etk_tree_rows_height_set (ETK_TREE(SL->tree), 60);

	edje_object_part_geometry_get(SP->frame, "scroll.frame",NULL,NULL,&w,&h);
	SL->col = etk_tree_col_new(ETK_TREE(SL->tree),NULL,w,0.0);
	etk_tree_col_model_add(SL->col, etk_tree_model_text_new());
	etk_tree_headers_visible_set(ETK_TREE(SL->tree),0);
	etk_tree_build(ETK_TREE(SL->tree));

	etk_box_append(ETK_BOX(SL->scroll), SL->tree, ETK_BOX_START, ETK_BOX_EXPAND_FILL, 0);

	etk_container_add(ETK_CONTAINER(SL->embed), SL->scroll);

	if (SP) {
		edje_object_part_swallow(SP->frame, "scroll.frame", etk_embed_object_get(ETK_EMBED(SL->embed)));
	}
	etk_widget_show_all(SL->embed);
	return SL;
}

